# This is a simple tool to procedurally generate a three-color gradient on a series of faces.
# Gradientator by steve r is licensed under a Creative Commons Attribution-NonCommercial-ShareAlike 3.0 Unported License.
#
# Furthermore, this software is provided "as is" and without any express or implied warranties, including, without
# limitation, the implied warranties of merchantability and fitness for a particular purpose.
#
# For more information on this plugin, please visit http://forums.sketchucation.com/viewtopic.php?f=323&t=45951
#
# Gradientator v1.21
#
# Version notes:
#
# v1.21:
# - Added version notes
# - Made WebDialog resizable, and taller.
#
# v1.2:
# - Changed variables to instance variables
# - Modified the way colors are calculated; guarantees the first, middle, and last colors will be as chosen by the user.
# - Changed the Inputbox to a WebDialog, with procedurally generated drop-down boxes.
#
# v1.1:
# - Fixed a ton of bad coding practices (wrapped plugin in a module, etc)
# - Added an inputbox at the beginning to allow the user to select the three colors in the gradient
# - Added "Distance from Origin" as a gradient method. Much better results! See images.
#
# v1.0:
# - Basic random gradient
# - Red-Yellow-Lime colors only

require 'sketchup.rb'

module SR_Gradientator

	# Setting VERBOSE to nil is a simple speed boost tip from Dan Rathbun on the Sketchucation forums.
	
	$VERBOSE = nil

	def self.openwindow()
		
		# Version check code adopted from Google.
	
		@version_required = 7.0

		if (Sketchup.version.to_f < @version_required)
			
			UI.messagebox("You must have Sketchup " + @version_required.to_s + " to run this plugin. Visit sketchup.google.com to upgrade.")
			return
			
		end
		
		@model = Sketchup.active_model
		
		# Combined with commit_operation later on, makes it so 'Undo' undoes the entire operation. The Disable_UI flag is set to True (this prevents SketchUp from updating windows as the plugin runs) as this is a recommended speed boost, as described by thomthom on the Sketchucation forums.
		
		@model.start_operation "Gradientation", true
		
		# Cycle through selection, add things that happen to be faces to a collection.
		
		@faces = @model.selection.select{ |e| e.is_a?(Sketchup::Face) }
		
		# Returns an error if the user didn't select any faces, or if they only selected non-faces.
		
		if @faces.length == 0
			
			UI.messagebox("To use Gradientator, please select the faces in your model you would like to apply a gradient to, and then select this option again.  Feel free to use any selection method that will also select edges, as these edges will be automatically ignored by Gradientator.")
			return nil
			
		end
		
		# Set up a WebDialog to be used for user input later.
		# WebDialog options are title, scrollable, registry key title, width, height, left, top, resizable.
		
		@grad_dialog = UI::WebDialog.new("Gradientator", false, "", 280, 300, 260, 140, true)
		@grad_dialog.set_background_color("D3D3D3")
		
		# These are the commands that the WebDialog is allowed to perform; basically, either begin the gradient process or close the WebDialog window.
		
		@grad_dialog.add_action_callback("get_data") do |grad_dialog,action_name|

			if action_name == "begin_gradientating"
				
				@coloronename = @grad_dialog.get_element_value("color_one_name")
				@colortwoname = @grad_dialog.get_element_value("color_two_name")
				@colorthreename = @grad_dialog.get_element_value("color_three_name")
				@grad_dialog.close
				gradientate(@coloronename,@colortwoname,@colorthreename,@faces,@model)
				
			end
			
			if action_name == "close_this_window"
				
				@grad_dialog.close
				
			end
			
		end
		
		# Generates a string the comprises one complete drop-down box list of options, using style tags to define the background color of each option (so the user can see what the color is when picking).
		
		@colorarray = Sketchup::Color.names
		@htmlcolorlist = ""
		
		@colorarray.each{ |e| 
			
			@tempcolor = Sketchup::Color.new e
			@htmlcolorlist += "<option value='"+e+"' style=background-color:rgb("+@tempcolor.red.to_s+","+@tempcolor.green.to_s+","+@tempcolor.blue.to_s+")>"+e+"</option>" 
		
		}
		
		# This mess is effectively a complete HTML file to be used in the WebDialog.
		
		@grad_html = '<html><script type="text/javascript">function callRubyGrad() {query = "skp:get_data@" + "begin_gradientating";window.location.href = query;}function callRubyClose() {query = "skp:get_data@" + "close_this_window";window.location.href = query;}</script><center><br><br>Select Color 1:   <select id="color_one_name">'+@htmlcolorlist+'</select><br><br>Select Color 2:   <select id="color_two_name">'+@htmlcolorlist+'</select><br><br>Select Color 3:   <select id="color_three_name">'+@htmlcolorlist+'</select><br><br><input type="button" onclick="callRubyGrad()" value=" OK ">  <input type="button" onclick="callRubyClose()" value=" Cancel "><br><br></center></html>'
		
		@grad_dialog.set_html(@grad_html)
		@grad_dialog.show_modal()
		
	end # End of def self.openwindow()
	
	def self.gradientate(coloronename,colortwoname,colorthreename,faces,model)
		
		# Attempt to use the strings from the WebDialog to generate SketchUp Color objects. Returns an error if there's a problem, but since the drop-down lists are populated from SketchUp's internal Color Names list, that's not expected to show.
		
		begin
		
			@colorone = Sketchup::Color.new coloronename
			@colortwo = Sketchup::Color.new colortwoname
			@colorthree = Sketchup::Color.new colorthreename
			puts "Color 1: #{coloronename}, RGB: #{@colorone.red}, #{@colorone.green}, #{@colorone.blue}"
			puts "Color 2: #{colortwoname}, RGB: #{@colortwo.red}, #{@colortwo.green}, #{@colortwo.blue}"
			puts "Color 3: #{colorthreename}, RGB: #{@colorthree.red}, #{@colorthree.green}, #{@colorthree.blue}"
		
		rescue
			
			UI.messagebox("Gradientator did not recognize one of your colors!")
			
		end
		
		# Determine how rapidly to alter the colors between the main colors (colors one, two, and three, as chosen by the user).
		
		@redshifta = (@colortwo.red - @colorone.red)/((faces.length.to_f)/2)
		@redshiftb = (@colorthree.red - @colortwo.red)/((faces.length.to_f)/2)
		@greenshifta = (@colortwo.green - @colorone.green)/((faces.length.to_f)/2)
		@greenshiftb = (@colorthree.green - @colortwo.green)/((faces.length.to_f)/2)
		@blueshifta = (@colortwo.blue - @colorone.blue)/((faces.length.to_f)/2)
		@blueshiftb = (@colorthree.blue - @colortwo.blue)/((faces.length.to_f)/2)
		
		# Define the initial color, which will be equal to color one.
		
		@redbalance = @colorone.red.to_f
		@greenbalance = @colorone.green.to_f
		@bluebalance = @colorone.blue.to_f
		
		# This just keeps track of how far along we are in the gradient process.
		
		@counter = 1
		
		# Sorts the previously unordered faces object into an object ordered by distance from origin. Code concept provided by Thom Thom and sdmitch from the Sketchucation forums.
		
		@stack = {}
		faces.each{|face| @stack[face]=ORIGIN.distance(face.bounds.center)}
		faces = @stack.sort { |a,b| a[1] <=> b[1] }.map { |n| n[0] }
		
		# Un-comment these for debugging purposes, if needed:		
		#puts "Faces #{faces.length} half #{(faces.length.to_f/2).round}"
		#puts "Colors #{results[0]}, #{colorone}; #{results[1]}, #{colortwo}; #{results[2]}, #{colorthree}."
		#puts "A Shifts #{redshifta}, #{greenshifta}, #{blueshifta}"
		#puts "B Shifts #{redshiftb}, #{greenshiftb}, #{blueshiftb}"
		
		# Cycles through everything in the faces collection, applying the correct material to each.
		
		faces.each{ |f|
			
			# This is another line of debug code, lets you know exactly what is being applied to each face.
			#puts "face #{counter} applying #{redbalance}, #{greenbalance}, #{bluebalance}"
			
			f.material = [@redbalance.round, @greenbalance.round, @bluebalance.round]
			
			@counter += 1
			
			# If we're halfway through the gradient process, or at the end, we should apply colortwo or colorthree, respectively. If we're less than halfway, we should use the next color between colorone and colortwo. If we're more than halfway, we should use the next color between colortwo and colorthree.
			
			if @counter == (@faces.length.to_f/2).round
			
				@redbalance = @colortwo.red
				@greenbalance = @colortwo.green
				@bluebalance = @colortwo.blue
			
			elsif @counter < @faces.length.to_f/2
			
				@redbalance += @redshifta
				@greenbalance += @greenshifta
				@bluebalance += @blueshifta
			
			elsif @counter == faces.length
				
				@redbalance = @colorthree.red
				@bluebalance = @colorthree.blue
				@greenbalance = @colorthree.green
			
			else
			
				@redbalance += @redshiftb
				@greenbalance += @greenshiftb
				@bluebalance += @blueshiftb
				
			end
			
		}
		
		model.commit_operation
		
	end # End of def self.gradientate()

end # End of Module SR_Gradientator

# Add Gradientate option to plugins menu.

if not file_loaded?(File.basename(__FILE__))
	
	UI.menu("Plugins").add_item("Gradientate") { SR_Gradientator::openwindow() }
	
end

file_loaded(File.basename(__FILE__))   # Mark the script loaded.